/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/
#include "EricssonLabsCellIDDatabase.h"
#include "../Utilities/Convert.h"
#include <mavsprintf.h>
#include <MAUtil/PlaceholderPool.h>

EricssonLabsCellIDDatabase::EricssonLabsCellIDDatabase()
						  : mDownloader(NULL),
						    mJSONParser(NULL)
{}

EricssonLabsCellIDDatabase::~EricssonLabsCellIDDatabase()
{
	if(mDownloader != NULL)
		delete mDownloader;

	if(mJSONParser != NULL)
		delete mJSONParser;
}

MALocation& EricssonLabsCellIDDatabase::convertToLocation(MACellInfo* cellinfo)
{
	LOG("Starting Ericsson Labs lookup");
	if(mDownloader == NULL)
		mDownloader = new HttpDownloader(this);


	char urlbuffer[255];
	sprintf(urlbuffer, "http://cellid.labs.ericsson.net/json/lookup?cellid=%08x&mnc=%s&mcc=%s&lac=%04x&key=KII72PaWpsVriUwfCBrIVlDY4uFQFNM3WVTdjKYL",
			   cellinfo->cellId, cellinfo->mnc, cellinfo->mcc, cellinfo->lac);
	LOG("URL: %s", urlbuffer);

	mDownloader->beginDownloading(urlbuffer);
}

//DownloadListener Interface
void EricssonLabsCellIDDatabase::notifyProgress(Downloader* downloader, int downloadedBytes, int totalBytes)
{
	LOG("Looking up database");
}

bool EricssonLabsCellIDDatabase::outOfMemory(Downloader* downloader)
{
	LOG("Database OOM");
}

void EricssonLabsCellIDDatabase::finishedDownloading(Downloader* downloader, MAHandle data)
{
	LOG("Finished database lookup");
	//Parse the returned data
	if(mJSONParser == NULL)
	{
		mJSONParser = new JSONParser();
		mJSONParser->addListener(this);
	}

	mJSONData.clear();
	char buffer[maGetDataSize(data) + 1];
	maReadData(data, &buffer, 0, maGetDataSize(data));

	mJSONData.append(buffer, maGetDataSize(data));

	//Delete the data
	maDestroyObject(data);
	PlaceholderPool::put(data);

	LOG("Parsing the JSON data");
	//Parse the result
	mJSONParser->parseString(mJSONData);

	//Check that we we're supplied with an accuracy
	if(mLocation.horzAcc == 0)
		mLocation.horzAcc = 5000; //Assume an accuracy of 5 km

	//Return the result to the location bus
	APPCONTROLLER->getLocationBus()->receiveLocation(mLocation);
}

void EricssonLabsCellIDDatabase::downloadCancelled(Downloader* downloader)
{
	LOG("Database download cancelled");
}

void EricssonLabsCellIDDatabase::error(Downloader* downloader, int code)
{
	LOG("Database download error %d", code);
}


//JSON
void EricssonLabsCellIDDatabase::header(String& header)
{
  mJSONHeader = header;
}

void EricssonLabsCellIDDatabase::data(String& value)
{
  if(mJSONHeader == "accuracy")
	  mLocation.horzAcc = Convert::toDouble(value);
  else if(mJSONHeader == "latitude")
	  mLocation.lat = Convert::toDouble(value);
  else if(mJSONHeader == "longitude")
	  mLocation.lon = Convert::toDouble(value);
}

void EricssonLabsCellIDDatabase::arrayStart()
{
}

void EricssonLabsCellIDDatabase::arrayEnd()
{
}

void EricssonLabsCellIDDatabase::objectStart()
{
}

void EricssonLabsCellIDDatabase::objectEnd()
{
}
