/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/
#ifndef ERICSSONLABSCELLIDDATABASE_H_
#define ERICSSONLABSCELLIDDATABASE_H_


#include <MAUtil/Environment.h>
#include "../Utilities/HttpDownloader.h"
#include "../Utilities/JSONParser.h"
#include "CellIDDatabase.h"
#include <IX_CELLID.h>

//This is intended as a base class for different CellID Implementations to share
using namespace MAUtil;

class EricssonLabsCellIDDatabase : public CellIDDatabase,
								   public DownloadListener,
								   public JSONListener
{
	public:
		EricssonLabsCellIDDatabase();
        ~EricssonLabsCellIDDatabase();

        //DownloadListener Interface
		void notifyProgress(Downloader* downloader, int downloadedBytes, int totalBytes);
		bool outOfMemory(Downloader* downloader);
		void finishedDownloading(Downloader* downloader, MAHandle data);
		void downloadCancelled(Downloader* downloader);
		void error(Downloader* downloader, int code);

        //JSONListener Interface
		void header(String& header);
		void data(String& value);
		void arrayStart();
		void arrayEnd();
		void objectStart();
		void objectEnd();

		//CellIDDatabase Interface
		MALocation& convertToLocation(MACellInfo* cellinfo);

	private:
        MALocation mLocation;
        HttpDownloader* mDownloader;
        JSONParser* mJSONParser;

        String mJSONHeader;
        String mJSONData;
};


#endif /* ERICSSONLABSCELLIDDATABASE_H_ */
