/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/
#ifndef LOCATION_H_
#define LOCATION_H_

#include <maapi.h>
#include <MAUtil/Vector.h>
using namespace MAUtil;

//Provides interfaces between Moblet and screens which want location data


//An interface for classes which want to be informed of a change in location.
//When a new location is received, these classes can be told
class ILocationListener
{
  public:
    virtual void locationReceived(MALocation& location);
};

//An interface for a location manager.  ILocationListeners can register with
//ILocation classes to get location updates
class ILocationBus
{
  public:
    //Add a new listener to the bus
    virtual void addLocationListener(ILocationListener* l);

    //Remove a listener from the bus
    virtual void removeLocationListener(ILocationListener* l);

    //An ILocationProvider can send its location to the bus to be distributed.
    virtual void informListeners(MALocation& location);

};

//An interface for classes which provide location information.  Locations are returned to
//the LocationBus
class ILocationProvider
{
  public:

	//Start getting location until told to stop
	virtual bool startLocation() = 0;

	//Stop getting location
	virtual void stopLocation() = 0;
};

#endif /* LOCATION_H_ */
