/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#include "LocationBus.h"
#include <madmath.h>
#include "../std.h"

LocationBus::LocationBus()
{
}

LocationBus::~LocationBus()
{}

void LocationBus::addLocationListener(ILocationListener* l)
{
  mListeners.insert(l);
}

void LocationBus::removeLocationListener(ILocationListener* l)
{
  mListeners.erase(l);
}

void LocationBus::receiveLocation(MALocation& location)
{

	LOG("Received location");
	if(mLat == 0 && mLon == 0 && mAccuracy == 0)
	{
		//This is the first location we've got
		mLat = location.lat;
		mLon = location.lon;
		mAccuracy = location.horzAcc;

		informListeners(location);
	}
	//if the new reading has greater accuracy than the old one, then report it
	else if (location.horzAcc < mAccuracy)
			informListeners(location);
	else
	{
		//Test to see if the location is outside the range of the last location.
		double R = 6371; // km
		double dLat = toRad(location.lat - mLat);
		double dLon = toRad(location.lon - mLon);
		double a = sin(dLat/2) * sin(dLat/2) +
				cos(toRad(mLat)) * cos(toRad(location.lat)) *
				sin(dLon/2) * sin(dLon/2);
		double c = 2 * atan2(sqrt(a), sqrt(1-a));
		double d = R * c; //d is the distance between the old location and the new.

		if(d > mAccuracy)
			informListeners(location);
	}
}

double LocationBus::toRad(double input)
{
	return (M_PI/180)*input;
}

void LocationBus::informListeners(MALocation& location)
{
	mLat = location.lat;
	mLon = location.lon;
	mAccuracy = location.horzAcc;

	for(Set<ILocationListener*>::Iterator i = mListeners.begin(); i != mListeners.end(); i++)
		(*i)->locationReceived(location);
}
