/*
 * CellIDProvider.cpp
 *
 *  Created on: 7 May 2010
 *      Author: sjp
 */
#include "CellIDProvider.h"
#include <mavsprintf.h>
#include <MAUtil/Util.h>
#include "../LocationBus/LocationBus.h"
#include "../std.h"

CellIDProvider::CellIDProvider()
			  : cellinfo(NULL),
				mDatabase(NULL),
				mTiming(false)
{
}

CellIDProvider::~CellIDProvider()
{
	if(cellinfo != NULL)
		delete cellinfo;

	//TODO: Think about whether CellIDProvider should delete the database when it releases...
	//PRO: It allows you to instance it in the constructor of the CellIDProvider, e.g new setCellIDDatabase(new CellIDDatabase());
	//CON: It won't stop you sharing a database across many cellIDProviders, but it will cause a runtime error.
	//		However, you've probably only got one CellIDProvider
	if(mDatabase != NULL)
		delete mDatabase;
}

//Start getting location
bool CellIDProvider::startLocation()
{
	if(mTiming)
		return true;

    Environment::getEnvironment().addTimer(this, 5000, 0);
    mTiming = true;

    LOG("CellIDProvider timer has started");
	return getLocation();
}

void CellIDProvider::stopLocation()
{
	if(mTiming)
	{
		Environment::getEnvironment().removeTimer(this);
		mTiming = false;
	}
}

//The timer can call back periodically to get a new CellID
void CellIDProvider::runTimerEvent()
{
	LOG("CellIDProvider timer event has fired");
	getLocation();
}

bool CellIDProvider::getLocation()
{
	if(cellinfo == NULL)
		cellinfo = new MACellInfo();

	LOG("CellIDProvider::getLocation - calling getCellInfo()");
	//Call the concrete implementation to get the cell id
	if(getCellInfo(cellinfo) < 0)
	{
		LOG("Cell ID Provider reported failure");
		return false; //CellId not supported on the platform
	}
	else
	{
		LOG("Got the Cell ID");
		LOG("Old CID: %8x New CID: %8x", mCellId, cellinfo->cellId);
	  //Check to see if we've moved to a different cell
      if(cellinfo->cellId != mCellId)
      {
    	  mCellId = cellinfo->cellId;
    	  LOG("Its a new Cell ID");
        //Get the position of the cell
    	  LOG("Calling database");
    	  if(mDatabase != NULL)
    		  mDatabase->convertToLocation(cellinfo);
      }
	}
 	return true;
}

void CellIDProvider::setCellIDDatabase(CellIDDatabase* ciddb)
{
	mDatabase = ciddb;
}

