/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#include "SonyEricssonCellIDProvider.h"
#include "../std.h"
#include "../Utilities/Convert.h"
#include <mastdlib.h>
#include <mavsprintf.h>

int SonyEricssonCellIDProvider::getCellInfo(MACellInfo* cellinfo)
{
	LOG("Getting SE Cell Info");
	int ret;
	bool success = false;

	char buffer[12];
	char* bptr = buffer;

	//Get the CellID
	if(maGetSystemProperty("com.sonyericsson.net.cellid", bptr, 12) > 0)
	{
		LOG("Got a cellid '%s'", bptr);
		cellinfo->cellId = Convert::hexToInt(bptr);
		memset(bptr, 0, 12);

		success = true;
	}

	if(success && maGetSystemProperty("com.sonyericsson.net.lac", bptr, 12) > 0)
	{
		LOG("Got a lac '%s'", bptr);
		cellinfo->lac = Convert::toInt(buffer);

		memset(bptr, 0, 12);
	}
	else
		success = false;

	if(success && maGetSystemProperty("com.sonyericsson.net.mcc", bptr, 4) > 0)
	{
		LOG("Got a mcc '%s'", bptr);
		//Convert the value from decimal to hex
		char temp[4];
		sprintf(temp, "%3x", Convert::toInt(buffer));
		strcpy(cellinfo->mcc, temp);
		memset(bptr, 0, 12);
	}
	else
		success = false;

	if(success && maGetSystemProperty("com.sonyericsson.net.mnc", bptr, 8) > 0)
	{
		LOG("Got a mnc '%s'", bptr);
		//Convert the value from decimal to hex
		char temp[8];
		sprintf(temp, "%7x", Convert::toInt(buffer));
		strcpy(cellinfo->mnc, temp);
		memset(bptr, 0, 12);
	}
	else
		success = false;

	if(success)
	{
		LOG("Success!");
		LOG("CellID: %8x", cellinfo->cellId);
		return 1;
	}
	else
	{
		LOG("Failure");
		return -1;
	}
}
