/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/
#ifndef CLOUDMADEPOIPROVIDER_H_
#define CLOUDMADEPOIPROVIDER_H_

#include <MAUtil/String.h>
#include <MAUtil/Set.h>
#include <MAP/Queue.h>
#include "PointsOfInterest.h"
#include "../Utilities/HttpDownloader.h"
#include "../Utilities/JSONParser.h"

using namespace MAUtil;
using namespace MAPUtil;
//Makes requests to the CloudMade Geocoding service.  You must provide a valid
//CloudMade API key for this work.

class CloudMadePoIProvider : public PoIProvider,
						     public DownloadListener,
						     public JSONListener
{
	public:
		CloudMadePoIProvider(const char* apikey);
		~CloudMadePoIProvider();

		//PoIProvider

		//After the request is fulfilled, the request will be deleted.
		void addRequest(PoIRequest* request);
		void addPoIProviderListener(PoIProviderListener* listener);
		void removePoIProviderListener(PoIProviderListener* listener);

		//DownloadListener
		void finishedDownloading(Downloader* downloader, MAHandle data);
		void notifyProgress(Downloader* downloader, int downloadedBytes, int totalBytes);
		void downloadCancelled(Downloader* downloader);
		void error(Downloader* downloader, int code);

		//JSONListener interface
	    void header(String& header);
	    void data(String& value);
	    void arrayStart();
	    void arrayEnd();
	    void objectStart();
	    void objectEnd();

	private:
		HttpDownloader mDownloader;
		Set<PoIProviderListener*> mListeners;
		Queue<PoIRequest> mRequests;
		JSONParser mJSONParser;
		String mApiKey;
		bool mIsBusy;
		int mReturnCount; //Tracks the number of PoI returned for this request
		PoIRequest* mCurrentRequest;
		PoIResponse mPoIResponse;

		void process();

		//Variables required for JSON processing
		int mObjectCount;
		double mFeatureLat;
		double mFeatureLon;
		String mFeatureName;
		String mFeatureType;
		bool mGotFeatures;
		String mHeader;
		int mCoPos; //Co-ordinate position in the array
};

#endif /* CLOUDMADEPOIPROVIDER_H_ */
