/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/
#ifndef POINTSOFINTEREST_H_
#define POINTSOFINTEREST_H_

#include <MAUtil/String.h>
using namespace MAUtil;

//This struct contains the data required to make a PoI request
struct PoIRequest
{
	double latitude;
	double longitude;
	int radius; //In metres
	String type; //Optional - you may specify the kind of PoI you are interested in e.g. "hotel"
	int max; //The maximum number of PoI you want to be returned
};

//This struct contains data about returned PoI
struct PoIResponse
{
	double latitude;
	double longitude;
	String name;
	String type;
};


//This interface must be inherited by classes wishing to query a PoIProvider
class PoIProviderListener
{
	public:
		virtual void poiFound(PoIResponse& response);
};

//This interface is for classes which will return Points of Interest (PoI) near
//a specific point

class PoIProvider
{
	public:
		virtual void addRequest(PoIRequest* request);

		virtual void addPoIProviderListener(PoIProviderListener* listener);
		virtual void removePoIProviderListener(PoIProviderListener* listener);
};

#endif /* POINTSOFINTEREST_H_ */
