/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#include "GoogleReverseGeocode.h"
#include <mavsprintf.h>
#include "../std.h"

GoogleReverseGeocode::GoogleReverseGeocode()
					: mDownloader(this)
{
	mJSONParser.addListener(this);
}

GoogleReverseGeocode::~GoogleReverseGeocode()
{
	if(mDownloader.isDownloading())
		mDownloader.cancelDownloading();
}

void GoogleReverseGeocode::addReverseGeocodeListener(ReverseGeocodeListener* listener)
{
	mListeners.insert(listener);
}

void GoogleReverseGeocode::removeReverseGeocodeListener(ReverseGeocodeListener* listener)
{
	mListeners.erase(listener);
}

void GoogleReverseGeocode::performReverseGeocode(double latitude, double longitude, bool isExact)
{
	mIsExact = isExact;
	mSentAddress = false;
	LOG("Starting reverse geocode");
	if(!mDownloader.isDownloading())
	{
		char buffer[255];
		char* bufferptr = buffer;
		sprintf(bufferptr, "http://maps.google.com/maps/api/geocode/json?latlng=%f,%f&sensor=true", latitude, longitude);
		LOG("RG: %s", bufferptr);
		mDownloader.beginDownloading(bufferptr);
		LOG("Started");
	}
}

//DownloadListener interface
void GoogleReverseGeocode::finishedDownloading(Downloader* downloader, MAHandle data)
{
	LOG("Reverse geocode download complete");
	LOG("Data Size %d bytes", maGetDataSize(data));
	//Read the data into a string
	char tc[maGetDataSize(data) + 1]; //Create a buffer to read the data into
	maReadData(data, &tc, 0, maGetDataSize(data));

	LOG("Starting JSON parser");
	mGotAddressHeader = false;
	String temp;
	temp.append(tc, strlen(tc));

	mJSONParser.parseString(temp);

	//Clean up the data object
	maDestroyObject(data);
}

void GoogleReverseGeocode::notifyProgress(Downloader* downloader, int downloadedBytes, int totalBytes)
{
	LOG("Reverse geocode downloading...");
}

void GoogleReverseGeocode::downloadCancelled(Downloader* downloader)
{
	LOG("Reverse geocode download cancelled");
}

void GoogleReverseGeocode::error(Downloader* downloader, int code)
{
	LOG("Reverse geocode download error: %d", code);
}

//JSONListener interface
void GoogleReverseGeocode::header(String& header)
{
	mGotAddressHeader = mIsExact && (header == "formatted_address");
	mGotLongNameHeader = !mIsExact && (header == "long_name");
}

void GoogleReverseGeocode::data(String& value)
{
	//Only send one address
	if(!mSentAddress)
	{
		//else if it isn't exact, then remember the values of 'long_name' until we reach the value
		//'locality'
		if(mGotLongNameHeader)
		{
			mLocality = value;
		}
		else if(!mIsExact && value == "locality")
		{
			//return the last "long_name" value we got
			informListeners(mLocality);
		}
		//If it is an exact position, then we want the first value of 'formatted_address'
		else if(mGotAddressHeader)
			informListeners(value);
	}
}

void GoogleReverseGeocode::arrayStart()
{}

void GoogleReverseGeocode::arrayEnd()
{}

void GoogleReverseGeocode::objectStart()
{}

void GoogleReverseGeocode::objectEnd()
{}

void GoogleReverseGeocode::informListeners(String& address)
{
	//This is the value we want
	for(Set<ReverseGeocodeListener*>::Iterator itr = mListeners.begin();
		itr != mListeners.end();
		itr++)
	{
		//Inform each listener
		LOG("This is the address:");
		LOG("%s", address.c_str());
		(*itr)->locationNameResolved(address);
		mGotAddressHeader = false;
		mSentAddress = true;
	}
}
