/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#ifndef GOOGLEREVERSEGEOCODE_H_
#define GOOGLEREVERSEGEOCODE_H_

#include "ReverseGeocode.h"
#include <MAUtil/String.h>
#include <MAUtil/Downloader.h>
#include <MAUtil/Set.h>
#include "../Utilities/JSONParser.h"
#include "../Utilities/HttpDownloader.h"

using namespace MAUtil;

//Connects to Google's Reverse Geocode service
class GoogleReverseGeocode : public ReverseGeocode,
							 public JSONListener,
						     public DownloadListener
{
	public:
		GoogleReverseGeocode();
		~GoogleReverseGeocode();

		void addReverseGeocodeListener(ReverseGeocodeListener* listener);
		void removeReverseGeocodeListener(ReverseGeocodeListener* listener);

		//Perform the reverse geocode
		void performReverseGeocode(double latitude, double longitude, bool isExact = true);

		//DownloadListener interface
		void finishedDownloading(Downloader* downloader, MAHandle data);
		void notifyProgress(Downloader* downloader, int downloadedBytes, int totalBytes);
		void downloadCancelled(Downloader* downloader);
		void error(Downloader* downloader, int code);

		//JSONListener interface
	    void header(String& header);
	    void data(String& value);
	    void arrayStart();
	    void arrayEnd();
	    void objectStart();
	    void objectEnd();

	private:
		HttpDownloader mDownloader;
		JSONParser mJSONParser;

		Set<ReverseGeocodeListener*> mListeners;

		bool mGotAddressHeader;
		bool mGotLongNameHeader;
		bool mSentAddress;

		bool mInStreetAddress;
		String mLocality; //Temporary storage of values until we can determine whether this is the 'locality' value or not
		bool mIsExact; //If this is true, then the lat/long came from an exact reading (e.g. GPS) rather
		//than Cell ID or something else.

		void informListeners(String& address);
};


#endif /* GOOGLEREVERSEGEOCODE_H_ */
