/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/
#include "MapScreen.h"
#include <MAUI/Image.h>
#include "../maheaders.h"
#include "../std.h"
#include <MAP/LonLat.h>
#include "mastdlib.h"

using namespace MAP;

MapScreen::MapScreen()
		 : mGeocode(NULL),
		   mHasReceivedLocation(false),
		   mPoIProvider(NULL)
{
	//Create the map widget
    mMap = new LayeredMap(0, 0, APPCONTROLLER->getScreenWidth(), APPCONTROLLER->getScreenHeight(), NULL);
    mMap->setMapSourceKind(MapSourceKind_OpenStreetMap);
    LonLat home;
    home.lat = 51.49663;
    home.lon = 0.00;
    mMap->enterMapUpdateScope();
    mMap->setCenterPosition(home);
    mMap->setMagnification(12);
    mMap->exitMapUpdateScope(true);
    mMap->setHasSmoothPanning(true);


    //Create the location notification widget
    mLabel = new Label(10, 10, APPCONTROLLER->getScreenWidth() - 20, 100, mMap);
    mLabel->setBackgroundColor(0xA0A0A0);
    mLabel->setDrawBackground(true);
    mLabel->setAutoSizeY(true);
    mLabel->setHorizontalAlignment(Label::HA_CENTER);
    mLabel->setVerticalAlignment(Label::VA_CENTER);
    mLabel->setPaddingLeft(10);
    mLabel->setPaddingRight(10);
    mLabel->setPaddingTop(5);
    mLabel->setPaddingBottom(5);
    mLabel->setMultiLine(true);
    mLabel->setCaption("Waiting for location...");

    //Adjust the position
	mLabel->update();
	mLabel->setPosition(10, APPCONTROLLER->getScreenHeight() - mLabel->getHeight() - 10);


    //Register for location events
    APPCONTROLLER->getLocationBus()->addLocationListener(this);

    //Create a layer
    mPoILayer = new MapLayer();

    //Add the layer to the map
    mMap->addLayer(mPoILayer);

    this->setMain(mMap);

}

MapScreen::~MapScreen()
{
	delete mLabel;
	delete mMap;

	if(mGeocode != NULL)
		delete mGeocode;
}

void MapScreen::show()
{
	Screen::show();
}

void MapScreen::setLocation(double lat, double lon, int zoom, String& caption)
{
	LonLat newloc;
	newloc.lat = lat;
	newloc.lon = lon;

	mMap->enterMapUpdateScope();
	mMap->setCenterPosition(newloc);
	mMap->setMagnification(zoom);
	mMap->exitMapUpdateScope(false);

	mLabel->setCaption(caption);
}

void MapScreen::setLocation(LonLat& ll)
{
	mMap->enterMapUpdateScope();
	mMap->setCenterPosition(ll);
	mMap->exitMapUpdateScope(false);
}

void MapScreen::setReverseGeocode(ReverseGeocode* reversegeocoder)
{
	mGeocode = reversegeocoder;
	mGeocode->addReverseGeocodeListener(this);
}

void MapScreen::setPoIProvider(PoIProvider* poiprovider)
{
	mPoIProvider = poiprovider;
	mPoIProvider->addPoIProviderListener(this);
}

void MapScreen::locationReceived(MALocation& location)
{
	LOG("Location received");
    //Location has been received from the location bus
	//Center the map on the new location
	LonLat newlocation;
	newlocation.lat = location.lat;
	newlocation.lon = location.lon;

	setLocation(newlocation);

	//Perform reverse geocode
	bool exact = location.horzAcc < 100; //Exact if within 100 meters

	LOG("Calling reverse geocoder");
	if(mGeocode != NULL)
		mGeocode->performReverseGeocode(location.lat, location.lon, exact);

	LOG("Calling poi provider");
	if(mPoIProvider != NULL)
	{
		PoIRequest* req = new PoIRequest();
		req->latitude = location.lat;
		req->longitude = location.lon;
		req->max = 10;
        int mag = mMap->getMagnification();

        //Calculate the size of the visible map in meters, so we can set search
        //parameters for PoI
        PixelCoordinate px1 = mMap->getCenterPositionPixels( );
        LonLat p1 = LonLat( px1 );
        PixelCoordinate px2 = PixelCoordinate( mag, px1.getX( ) + 1, px1.getY( ) + 1 );
        LonLat p2 = LonLat( px2 );
        double meterX1, meterY1;
        double meterX2, meterY2;
        p1.toMeters( meterX1, meterY1 );
        p2.toMeters( meterX2, meterY2 );
        double offsetX = meterX2 - meterX1;
        offsetX *= cos( fabs( p2.lat * PI / 180 ) );

		req->radius = (int)(offsetX * (APPCONTROLLER->getScreenWidth())); //The radius is set to what is visible on screen
		req->type = ""; //Any type will do, but you can change this to a value found in http://developers.cloudmade.com/wiki/geocoding-http-api/Object_Types
	//	LOG("Starting POI request");
		mPoIProvider->addRequest(req);
	}
}

void MapScreen::keyPressEvent(int keyCode)
{
	switch(keyCode)
	{
        case MAK_1:
        	mMap->scroll(SCROLLDIRECTION_WEST, false);
        	mMap->scroll(SCROLLDIRECTION_NORTH, false);
            break;
        case MAK_2:
        	mMap->scroll(SCROLLDIRECTION_NORTH, false);
            break;
        case MAK_3:
        	mMap->scroll(SCROLLDIRECTION_EAST, false);
        	mMap->scroll(SCROLLDIRECTION_NORTH, false);
            break;
        case MAK_4:
        	mMap->scroll(SCROLLDIRECTION_WEST, false);
            break;
        case MAK_6:
        	mMap->scroll(SCROLLDIRECTION_EAST, false);
            break;
        case MAK_7:
        	mMap->scroll(SCROLLDIRECTION_WEST, false);
        	mMap->scroll(SCROLLDIRECTION_SOUTH, false);
            break;
        case MAK_8:
        	mMap->scroll(SCROLLDIRECTION_SOUTH, false);
            break;
        case MAK_9:
        	mMap->scroll(SCROLLDIRECTION_EAST, false);
        	mMap->scroll(SCROLLDIRECTION_SOUTH, false);
            break;
		case MAK_STAR:
			mMap->zoomIn();
			mMap->updateMap();
			break;
		case MAK_HASH:
			mMap->zoomOut();
			mMap->updateMap();
			break;
	}
}

//ReverseGeocodeListener
void MapScreen::locationNameResolved(String& locationName)
{
	LOG("Location Name: %s", locationName.c_str());
	mLabel->setCaption(locationName);
	mLabel->update();
	mLabel->setPosition(10, APPCONTROLLER->getScreenHeight() - mLabel->getHeight() - 10);
}

//PoIProviderListener
void MapScreen::poiFound(PoIResponse& response)
{
	//Create a new marker to go on the map
	if(response.name.length() > 0)
	{
		PoI* poi = new PoI(0, 0, 100, 80, NULL);
		poi->getCaption()->setCaption(response.name);
		poi->setLonLat(response.longitude, response.latitude);
		mPoILayer->addPin(poi);
		LOG("Adding %s at %f : %f", response.name.c_str(), response.longitude, response.latitude);
		mMap->updateMap();
	}
}
