/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/
#ifndef _MAPSCREEN_H_
#define _MAPSCREEN_H_

#include <MAUI/Screen.h>
#include <MAUI/Label.h>
#include "../Widgets/LayeredMap.h"
#include "../Location.h"
#include "../ReverseGeocoders/ReverseGeocode.h"
#include "../PoI/PointsOfInterest.h"
#include "../Widgets/MapLayer.h"
#include "../Widgets/PoI.h"

using namespace MAUI;
using namespace MAP;

class MapScreen : public Screen,
				  public ILocationListener,
				  public ReverseGeocodeListener,
				  public PoIProviderListener
{
	public:
		MapScreen();
		~MapScreen();
		void keyPressEvent(int keyCode);

		void setLocation(LonLat& ll);
		void setLocation(double lat, double lon, int zoom, String& caption);

		//LocationListener interface
		void locationReceived(MALocation& location);

		//ReverseGeocode interface
		void locationNameResolved(String& locationName);

		//PoIProvider interface
		void poiFound(PoIResponse& response);

		void show();

		//Set the reverse geocoder to use
		void setReverseGeocode(ReverseGeocode* reversegeocoder);

		//Set the PoIProvider to use
		void setPoIProvider(PoIProvider* poiprovider);

	private:
		Label* mLabel;
		LayeredMap* mMap;
		ReverseGeocode* mGeocode;
		PoIProvider* mPoIProvider;
		MapLayer* mPoILayer;

		bool mHasReceivedLocation;
};

#endif	//_MAPSCREEN_H_
