/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#ifndef HTTPDOWNLOADER_H_
#define HTTPDOWNLOADER_H_

#include <MAUtil/Connection.h>
#include <MAUtil/Downloader.h>
#include <MAUtil/PlaceholderPool.h>
#include <MAUtil/Set.h>
using namespace MAUtil;


//This class performs an HTTP 1.0 GET operation, and will download data to an MAHandle.
//Unlike MAUtil::Downloader, this does not require the webserver to send a content-length
//header
class HttpDownloader : public Downloader
{
	public:
		HttpDownloader();
		HttpDownloader(DownloadListener* listener);
		~HttpDownloader();

		int beginDownloading(const char* url, MAHandle = NULL);
		void cancelDownloading();

		bool isDownloading();


		//HttpConnectionListener interface
		void httpFinished(HttpConnection* http, int result);
		void connectFinished(Connection* conn, int result);
		void connRecvFinished(Connection* conn, int result);
		void connWriteFinished(Connection* conn, int result);
		void connReadFinished(Connection* conn, int result);

	private:
		Set<DownloadListener*> mListeners;
		HttpConnection mConnection;
		MAHandle mData;
		MAHandle mTemp;
		MAHandle mSwap;
		bool mPreservemData;
		bool mIsRedirect;
		bool mSizeKnown;
		int mSize;
		String* mRedirectUrl;
		String* mConLen;

		void reportError(int errorcode);
		void reportProgress(int bytes);
		void reportFinish();
		void reportOOM();
		void reportCancelled();
};

#endif /* HTTPDOWNLOADER_H_ */
