/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#include "MapLayer.h"
#include <MAUtil/Graphics.h>
#include "../std.h"

MapLayer::MapLayer()
		: mVisible(true)
{
}

MapLayer::~MapLayer()
{
	//Delete pins
	Vector_each(MapPin*, itr, mPins)
		delete *itr;
}

void MapLayer::drawLayer(MapWidget* map)
{
	//LOG("Map Layer being drawn");
	if(mVisible)
	{
		//LOG("Drawing MapPins");

		Vector_each(MapPin*, itr, mPins)
		{
			MapPin* mp = *itr;
			//Get the position of this point in pixels
			//Get the world pixels
			PixelCoordinate pc = mp->getLonLat().toPixels(map->getMagnification());
			//LOG("World pixels %d : %d", pc.getX(), pc.getY());
			MAPoint2d point = map->worldPixelToWidget(pc);
			//LOG("Pixel position is %d : %d", point.x, point.y);

			Gfx_pushMatrix();

				//Transform to the position on screen
				Gfx_translate(point.x + mp->getXOffset(), point.y + mp->getYOffset());

				//Draw the widget as normal
				(*itr)->setEnabled(true);
				(*itr)->draw(true);
				(*itr)->setEnabled(false);

			Gfx_popMatrix();
		}

	}
}

bool MapLayer::isVisible()
{
	return mVisible;
}

void MapLayer::setVisible(bool v)
{
	mVisible = v;
}

void MapLayer::addPin(MapPin* pin)
{
	mPins.add(pin);
}

