/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#include <MAUtil/Moblet.h>
#include <MAUI/Font.h>
#include <MAUI/WidgetSkin.h>
#include "std.h"
#include "MAHeaders.h"
#include "Utilities/Convert.h"
#include "AppLoader.h"
#include <MAUI/Engine.h>

#include "Screens/MapScreen.h"

// Location providers - uncomment as necessary
#include "LocationProviders/TestProvider.h"
//#include "LocationProviders/GPSProvider.h"
//#include "LocationProviders/SymbianS60CellIDProvider.h"
//#include "LocationProviders/SonyEricssonCellIDProvider.h"
#include "LocationProviders/TestCellIDProvider.h"

// Cell ID Databases - uncomment as necessary
//#include "CellIDDatabases/EricssonLabsCellIDDatabase.h"
#include "CellIDDatabases/GoogleCellIDDatabase.h"

// Reverse Geocoders - uncomment as necessary
#include "ReverseGeocoders/GoogleReverseGeocode.h"

// PoI Provider
#include "PoI/CloudMadePoIProvider.h"

//Location bus
#include "LocationBus/LocationBus.h"

using namespace MAUtil;
using namespace MAUI;

class LocationMoblet :  public Moblet
{
	public:
		LocationMoblet()
		{
		    APPCONTROLLER->addFont("default", new Font(FONT));
    		Engine& engine = Engine::getSingleton();
			engine.setDefaultFont(APPCONTROLLER->getFont("default"));

			APPCONTROLLER->setMoblet(this);

			//Load the screens - no knowledge of screens required at buildtime
			AppLoader* loader = new AppLoader();
			loader->load();
			delete loader;

			//Start getting locations

			//To enable GPS, uncomment the next two lines
			//GPSProvider* gps = new GPSProvider();
			TestProvider* gps = new TestProvider();
			gps->startLocation();

			//Select which cell id providers you want by changing the next line
			//CellIDProvider* cellid = new SonyEricssonCellIDProvider();
			//CellIDProvider* cellid = new TestCellIDProvider();
			//CellIDProvider* cellid = new SymbianS60CellIDProvider();

			//CellIDProviders need a database, so set yours here
			//cellid->setCellIDDatabase(new GoogleCellIDDatabase());

			//Start receiving CellID locations
			//LOG("Starting CellID");
			//cellid->startLocation();

			//Get the MapScreen
			MapScreen* mapScreen = (MapScreen*)APPCONTROLLER->getScreen(MAPSCREEN);

			//Set the reverse geocoder
			mapScreen->setReverseGeocode(new GoogleReverseGeocode());

			//Set the points of interest provider
			mapScreen->setPoIProvider(new CloudMadePoIProvider("BC9A493B41014CAABB98F0471D759707"));

			//Show the map to the user
			APPCONTROLLER->show(MAPSCREEN);
		}

		~LocationMoblet()
		{
			delete APPCONTROLLER;
		}

		void keyPressEvent(int keyCode)
		{
		}

		void pointerPressEvent(MAPoint2d point)
		{
		}

	    void customEvent(const MAEvent& event)
	    {
	      //Check to find if this is a location event
	      if(event.type == EVENT_TYPE_LOCATION)
	      {
	        LOG("Location received");
	        MALocation& loc = *(MALocation*)event.data;
	        APPCONTROLLER->getLocationBus()->receiveLocation(loc);
	      }
	    }
	private:
};

extern "C" int MAMain()
{
	Moblet::run(new LocationMoblet());
	return 0;
};
